#!/usr/bin/perl
# dia2sql.pl - SMDP version $Revision: 1.11 $
# branched of the dia2sql.pl - version 1.2
#
# dia2sql.pl version 1.2 is:
# Copyright (c) 2001 by Alexander Troppmann
# http://www.cocktaildreams.de - talex@karyopse.de
#
# This program releases under the GNU Public License;
# You can redistribute it or modify it under the terms of GPL.
#
# Required stuff:
# - First get the expat library (> v1.95.0) as RPM package or from
#   http://www.jclark.com/xml/expat.html which is a XML1.0 parser written in C.
# - Second install XML::Parser from CPAN, just type in at your shell prompt:
#      root@localhost:~ > perl -MCPAN -e 'install XML::Parser'

# load necessary perl-modules
use XML::Parser;
use Getopt::Long;

$VERSION = 'dia2sql.pl v1.3 SMDP-Release';


###########################################################
## CONFIG - MAKE ANY CHANGES HERE
## all these things can be turned on on the command-line

$DEBUG = 0;			# 1 if you want to get debug messages to STDERR
$CREATECOMMENTS = 0;		# 1 if you want to get comments like mysqldump does
$MYSQL = 1;			# choose either MySQL or PostgreSQL support
$POSTGRESQL = 0;		# by setting 0 and 1 values here
$MSSQL = 0;                 	# 
$CREATEDROPTABLES = 0;		# 1 if you want to have "DROP TABLE" statements
$INDECES = 0;               	# 1 if you want INDECES created.
$CONVERT = 0;               	# convert datatypes to target-db
$DATATYPES_FILE = "datatypes.txt"; # the default datatypes-file
$INPUT = "mysql";           	# default for input-type

###########################################################
## VARIABLES

$table = 0;			# 1 if inside table
$tablename = 0;			# 1 if tablename tag detected
$column = 0;			# 1 if inside column
$columnname = 0;		# 1 if columnname detected
$columntype = 0;		# 1 if datatype for column detected
$columnvi = 0;			# 1 if "visibility" for primary key definitions
$columnvalue = 0;		# 1 if value attribute detected

undef($myTablename);		# current table
undef(%myTableContents);	# hash with columns for a certain table
undef(%myKeyMarker);		# marker for primary keys
undef(%myIndexMarker);		# marker for indexed column
undef(%myPrimaryKeys);		# hash with primary keys for a certain table
undef(%myIndexColumns);		# columns for index
undef($myColumnvalue);		# default value for datatype
undef($myTableUsesForeignKeys);	# is my current table using foreign keys?
undef($myTableUsesIndizes);	# is my current table using indexed columns?
undef(@myTablesIncludingFKs);	# build these tables later
undef(@myTables);		# build tables first (cause they're referenced)

###########################################################
## INIT

## get our command-line args
$results = &GetOptions( "mysql", \$mysql,                     	# generate mysql-output
                        "postgresql", \$postgresql,           	# generate postgresql-output
                        "mssql", \$mssql,               	# generate ms-sql-output
                        "debug", \$DEBUG,               	# we want debug
                        "with-comments", \$CREATECOMMENTS,    	# we want comments
			"with-create", \$CREATEDROPTABLES,	# we want creates
			"absolute-clean", \$CLEAN,      	# we want drops
                        "create-indices", \$INDECES,    	# we want indeces
                        "database=s" => \$database,		# the database we want, defaults to 'smdp'
                        "convert", \$CONVERT,         		# convert datatypes to target-database
                        "datatypes=s" => \$DATATYPES_FILE,	# our datatypes file
                        "input=s" => \$INPUT,          	  	# which input
			"with-html-output", \$HTML_OUTPUT,	# give output to stdout from sql-file
                        "h", \$usage);                  	# I need help! 
                        

# check if the (l)user cried for help or gave us wrong args.
if((!$results) || ((defined($mysql)) && (defined($postgresql)) && defined($mssql)) || (defined($usage))) {   
    &usage();
}

# working through the args.

if(defined($postgresql)) {   
    $POSTGRESQL = 1;
    $MYSQL      = 0;
    $MSSQL      = 0;
	$comment_char = "--";
}

if(defined($mysql)) {
    $MYSQL 	= 1;
    $POSTGRESQL = 0;
    $MSSQL = 0;
	$comment_char = "#";
}

if(defined($mssql)) {
    $MSSQL = 1;
    $MYSQL = 0;
    $POSTGRESQL = 0;
	$comment_char = "--";
}

if(defined($database)) {
    $my_database = $database;
} else {
    $my_database = "smdp";
}

if(defined($CLEAN) && defined($CREATEDROPTABLES)) {
    undef($clean_counter); # we undef() the counter we'll later use to just
                           # print once the drop-table :> *sigh* lazy fkr
}

if ($INPUT eq "") {
    &usage();
}

if(($CONVERT) && (! -e $DATATYPES_FILE)) {
    &usage();
}

if(@ARGV < 1) {
    &usage();
} else {
    $infile = shift;		# input file, Dia XML formatted
    $outfile = shift;		# output file, filled with SQL statements
	
    if($outfile) {
       open(STDOUT, ">$outfile") or die "$outfile: $!n";
    }
	
}

# done with args
##

if($CONVERT) {   
    open(DATATYPES,"<$DATATYPES_FILE") or die "failed to open $DATATYPES_FILE\n";
    while(<DATATYPES>) {
        if($_ =~ /^#.*/) {
            next;
        }
        @fields = split /,/, $_;
        chomp($fields[0]);
        chomp($fields[1]);
        chomp($fields[2]);
        push(@mysqlTypes,$fields[0]);
        push(@pgsqlTypes,$fields[1]);
        push(@mssqlTypes,$fields[2]);
    }
    close(DATATYPES);
}

if($INPUT =~ /mysql/) {
    @inputTypes = @mysqlTypes;
} elsif ($INPUT =~ /pgsql/) {
    @inputTypes = @pgsqlTypes;
} elsif ($INPUT =~ /mssql/) {
    @inputTypes = @mssqlTypes;
}

if ($INPUT eq "") {
    &usage();
}

if($MYSQL) {
    $DROPTABLE = "DROP TABLE IF EXISTS";    # DROP TABLE statement to be used
    } elsif($POSTGRESQL) {
       $DROPTABLE = "DROP TABLE";
       $CREATEFKSFROMDBNAMING = 1;             # creates REFERENCE for FK_ columns
    } elsif($MSSQL) {
    } else {
        die "ERROR: You didn't choose a database!\n";
}


if($MYSQL) {
	if($DEBUG) { print STDERR "Creating SQL output for MySQL\n"; }
} elsif($POSTGRESQL) {
	if($DEBUG) { print STDERR "Creating SQL output for PostgreSQL\n"; }
} elsif($MSSQL) {
    if($DEBUG) { print STDERR "Creating SQL output for MS-SQL\n"; }
}


## init xml parser and parse input file
#
$parser = new XML::Parser(Style => 'Stream');
$parser->parsefile($infile);


## create sql output
#
&createSql();


## cleanup and exit
#
if($outfile) {
	close(STDOUT);
}

exit;


###########################################################
## SUB ROUTINES

## called if parser enters new tag
#
$prefix = '';				# patch for Dia v0.83 by Georges Khaznadar

sub StartTag {
	
	my $p = shift;			# parser context
	$ctag = shift;			# name of this tag element
	%attr = %_;				# hash with attributes
	
	if($ctag eq 'dia:diagram') {
	    # then all other tags will be prefixed with dia:
		# (patch for Dia v0.83 by Georges Khaznadar)
	    $prefix = 'dia:';
	}
	
	if($ctag eq $prefix.'object' and $attr{'type'} eq 'UML - Class') {
        $table = 1;
        $myTableUsesForeignKeys = 0;
        $myTableUsesIndizes = 0;
    } elsif($ctag eq $prefix.'composite' and $attr{'type'} eq 'umlattribute' and $table) {
        $column = 1;
	} elsif($table and $ctag eq $prefix.'attribute' and $attr{'name'} eq 'name' and !$column) {
		$tablename = 1;
	} elsif($column and $ctag eq $prefix.'attribute' and $attr{'name'} eq 'name' and !$tablename) {
		$columnname = 1;
	} elsif($column and $ctag eq $prefix.'attribute' and $attr{'name'} eq 'value' and $column) {
		$columnvalue = 1;
	} elsif($column and $ctag eq $prefix.'attribute' and $attr{'name'} eq 'type') {
		$columntype = 1;
	} elsif($column and $ctag eq $prefix.'attribute' and $attr{'name'} eq 'visibility') {
		$columnvi = 1;
	} elsif($columnvi and $ctag eq $prefix.'enum') {
		if(($attr{'val'} == 2) && (!$POSTGRESQL)) { $myKeyMarker = 1; }		# primary key found
		else { $myKeyMarker = 0; }
		if(($attr{'val'} == 1) && ($INDECES)) { $myIndexMarker = 1; }		# index column found
		else { $myIndexMarker = 0; }
	}
	
}

## called if parser leaves a tag
#
sub EndTag {
	
	my $p = shift;
	$ctag = shift;
	
	if($ctag eq $prefix.'object' and $table) {
		
		$table = 0;
		$myTableUsesIndizes = 0;
		
		if($myTableUsesForeignKeys) {
			push @myTablesIncludingFKs, $myTablename;
			$myTableUsesForeignKeys = 0;
			if($DEBUG) { print STDERR "Table '$myTablename' has foreign keys\n"; }
		} else {
			push @myTables, $myTablename;
			if($DEBUG) { print STDERR "Table '$myTablename' (may be referenced by other tables)\n"; }
		}
		
	} elsif($ctag eq $prefix.'composite' and $column) {
		
		$column = 0;
		my $sql = "$myColumnname $myColumntype";
		
		if($myKeyMarker) {
			push @{$myPrimaryKeys{$myTablename}}, "$myColumnname";
			if($MYSQL) {
				$sql .= " NOT NULL";
			}
		}
		
		if($myIndexMarker and $POSTGRESQL) {
			push @{$myIndexColumns{$myTablename}}, "$myColumnname";
			$myTableUsesIndizes = 1;
			if($DEBUG) { print STDERR "Indexed column found!\n"; }
		}
		
		if($myColumnvalue) {
			$sql .= " DEFAULT '$myColumnvalue'";
		}
		
		if($CREATEFKSFROMDBNAMING) {
			if($myColumnname =~ /^FK_/) {
				$myColumnname =~ /([A-Za-z0-9]+)_([A-Za-z0-9]+)$/;
				my ($table, $column) = ($1, $2);
				$sql .= " REFERENCES $table";
				$myTableUsesForeignKeys = 1;
				if($DEBUG) { print STDERR "Foreign key found!\n"; }
			}
		}
		
		undef($myColumnvalue);
		
		push @{$myTableContents{$myTablename}}, $sql;
		if($DEBUG) { print STDERR "Added new column data \"$sql\"\n"; }
		
	} elsif($ctag eq $prefix.'attribute' and $tablename) {
		$tablename = 0;
	} elsif($ctag eq $prefix.'attribute' and $column and $columnname) {
		$columnname = 0;
	} elsif($ctag eq $prefix.'attribute' and $column and $columnvalue) {
		$columnvalue = 0;
	} elsif($ctag eq $prefix.'attribute' and $column and $columntype) {
		$columntype = 0;
	} elsif($ctag eq $prefix.'enum' and $column and $columnvi) {
		$columnvi = 0;
	}
	
}


## called for text between any tags
#
sub Text {
	
	$text = $_;
	
	if($text =~ /^\s+$/) { return; }		# skip whitespaces
	
	if($ctag eq $prefix.'string' and $tablename) {
		$text =~ s/(^#|#$)//g;				# remove hash characters
		$myTablename = $text;
		if($DEBUG) { print STDERR "\nTable: $myTablename\n"; }
	} elsif($ctag eq $prefix.'string' and $columnname) {
		$text =~ s/(^#|#$)//g;              # remove hash characters
		$myColumnname = $text;
		if($DEBUG) { print STDERR "Columnname: $myColumnname\n"; }
	} elsif($ctag eq $prefix.'string' and $columnvalue) {
		$text =~ s/(^#|#$)//g;
		$text =~ s/('|")/\\$1/sg;
		$myColumnvalue = $text;
		if($DEBUG) { print STDERR "Columnvalue: $myColumnvalue\n"; }
	} elsif($ctag eq $prefix.'string' and $columntype) {
		$text =~ s/(^#|#$)//g;              # remove hash characters
		$myColumntype = $text;
        
        # this constructs converts the datatypes between databases.
        if($CONVERT) {
            $counter = 0;
            # converting to postgres
            if($POSTGRESQL) {
                if(($myColumntype =~ /.*AUTO_INCREMENT.*/) || ($myColumntype =~ /.*IDENTITY.*/))   {   
                    $newColumntype = "SERIAL";
                } else {
                    foreach(@inputTypes) {   
                        if($_ eq $myColumntype) {   
                            $newColumntype = $pgsqlTypes[$counter];
                        }
                        $counter++;
                    }
                }
            }
        
            # converting to mysql
            if($MYSQL) {
                if(($myColumntype =~ /.*SERIAL.*/) || ($myColumntype =~ /.*IDENTITY.*/))   { 
                    $newColumntype = "BIGINT(20) UNSIGNED AUTO_INCREMENT NOT NULL";                
                } else {
                    foreach(@inputTypes) {
                        if($_ eq $myColumntype) {
                            $newColumntype = $mysqlTypes[$counter];
                        }
                        $counter++;
                    }
                }
            }

            #converting to ms-sql
            if($MSSQL) {
                if(($myColumntype =~ /.*AUTO_INCREMENT.*/) || ($myColumntype =~ /.*SERIAL.*/))  {
                        $newColumntype = "INT IDENTITY";
                } else {
                    foreach(@inputTypes) {
                        if($_ eq $myColumntype) {
                            $newColumntype = $mssqlTypes[$counter];
                        }
                        $counter++;
                    }
                }
           }
       }
        
       if(defined($newColumntype)) {
                $myColumntype = $newColumntype;
                undef($newColumntype);
        }
    	if($DEBUG) { print STDERR "Columntype: $myColumntype\n"; }
	}
	
}


## create sql output
#
sub createSql {
	
	my($columns,$keys,$sql,$date);
	
	if($DEBUG) { print STDERR "\nWriting SQL statements...\n"; }
	
	if($CREATECOMMENTS) {
		$date = `date`; chop($date);
		print "$comment_char Created by $VERSION (".$date.")\n\n";
	}
	
	if($HTML_OUTPUT) {
		if($MSSQL) {
			print "PRINT '<HTML><HEAD><TITLE>SQL - Script - by dia2sql.pl</TITLE></HEAD><BODY><H1>SQL setup log</H2>'\n\n";
		}
	}
	
	if($DEBUG) { print STDERR "\nFirst build tables referenced by other tables...\n"; }
	foreach(@myTables) {
		&buildTable($_);
	}
	
	if($DEBUG) { print STDERR "\nBuild tables including foreign keys...\n"; }
	foreach(@myTablesIncludingFKs) {
		&buildTable($_);	
	}
	
	if($DEBUG) { print STDERR "Done!\n\n"; }
	
}


## build sql table
#
sub buildTable($) {
	
	my $tablename = shift;
    
	if(defined($sql)) {
        undef($sql);
    }
	
	if($DEBUG) { print STDERR "Working on '$tablename':\n"; }
	
	if(($CLEAN) && ($clean_counter == 0)) {
		if($DEBUG) { print STDERR "-> Creating DROP DATABASE statement\n"; }
		if($CREATECOMMENTS) { print "$comment_char drop statement to kill database.\n"; }
		
		if($HTML_OUTPUT) {
			if($MSSQL) {
				print "PRINT '<LI>Dropping $my_database  if it exists<BR>'\n\n";
			}
		}
		
		if($MSSQL) {
			print "USE MASTER\n".
			      "GO\n".
			      "IF EXISTS (SELECT name FROM sysdatabases WHERE name = N'$my_database')\n".
				  "\tDROP DATABASE $my_database\n".
				  "GO\n\n";
		} elsif($MYSQL) {
			print "DROP DATABASE IF EXISTS $my_database;\n";
		} elsif($POSTGRESQL) {
			print "DROP DATABASE $my_database;\n";
		}

	}

	if($CREATEDROPTABLES) {
		 if($clean_counter == 0) {
		 	if($DEBUG) { print STDERR "-> Creating CREATE DATABASE statement\n"; }
			if($CREATECOMMENTS) { print "$comment_char create database statement\n"; }
			
			if($HTML_OUTPUT) {
	            if($MSSQL) {
	                print "PRINT '<LI>Creating tables'\n\n";
	            }
	        }


			# i know these almost alike...no I'm not on crack, MS-SQL has no ';' at the end
			if($MSSQL) {
				print "CREATE DATABASE $my_database\n".
				      "GO\n".
					  "USE $my_database\n".
					  "GO\n\n";
			} else {
				print "CREATE DATABASE $my_database;\n";
				if($MYSQL) {
					print "USE $my_database;\n\n";
				} elsif($POSTGRESQL) {
					print "\\c smdp\n\n";
				}
			}
		 }
	
		 if($CREATECOMMENTS) {
		 	print "\n$comment_char\n$comment_char Table structure for table '$tablename'\n$comment_char\n\n";
		 }
			
		 if($DEBUG) { print STDERR "-> Creating DROP TABLE statement\n"; }
             # ms-sql has a different way of creating a table
	     if($CLEAN) {
		 	if($MSSQL) {
                print "if exists (select * from dbo.sysobjects where id = object_id(N'[$tablename]')\n".
                      "and OBJECTPROPERTY(id, N'IsUserTable') = 1)\n".
                      "\tdrop table [$tablename]\n".
                      "GO\n\n";
            } else {
                print "$DROPTABLE $tablename;\n";
            }
		}
	} else {
		if($CREATECOMMENTS) {
			print "\n$comment_char\n$comment_char Table structure for table '$tablename'\n$comment_char\n\n";
		}
	}

	$clean_counter++; # make sure, we only do this the first time

	if($DEBUG) { print STDERR "-> Collect table columns\n"; }
	my $columns = join(",\n\t",@{$myTableContents{$tablename}});
	my $keys = "PRIMARY KEY(".join(",",@{$myPrimaryKeys{$tablename}}).")";
	
	if($DEBUG) { print STDERR "-> Create CREATE TABLE statement\n"; }
	
	if($CREATEDROPTABLES) {
		$sql = "CREATE TABLE $tablename (\n\t".$columns;
	} 
	
	if(@{$myPrimaryKeys{$tablename}}) {
		if($DEBUG) { print STDERR "-> Adding PRIMARY KEY definitions\n"; }
		$sql .= ",\n\t".$keys."\n);\n\n";
	} else {
		$sql .= "\n);\n\n";
	}
	
	print $sql;
	undef($sql);
	
	if(@{$myIndexColumns{$tablename}}) {			# add indexed columns
		foreach $column (@{$myIndexColumns{$tablename}}) {
			if($DEBUG) { print STDERR "-> Adding INDEX for $tablename ($column)\n"; }
			$sql .= "CREATE INDEX ".$column."_idx ON $tablename ($column);\n";
		}
	}
	
	print "$sql\n";
	
}

## show usage
#
sub usage() {
    print STDOUT <<E_O_USAGE;

$VERSION 
original author: 2001 by Alexander Troppmann
heavily patched by the SMDP-Group

Usage:  dia2sql.pl [options] infile.dia [outfile.sql]
Options may be:   
  -h                    print this help.
  --mysql               create SQL for mysql
  --postgresql          create SQL for postgresql
  --mssql               create SQL for ms-sql-server
  --debug               enter debug-mode
  --with-comments       write comments
  --with-create         write create statements for tables
  --with-html-output    writes print-statements with html-stuff 
                        into sql-script. only useful for 
						database-parsers that will display that 
						like isql.exe from MS-SQL Server.
  --absolute-clean      write drop/create statements for the database and
                        all tables. use it wisely!
  --create-indices      create indeces (postgres-only)
  --convert             convert datatypes to fit target db
  --database=<database> name of the database to be used for the create-statement
                        if not given, defaults to smdp.
  --datatype=<file>     file needs to hold the mapping of the datatypes.
                        (defaults to datatypes.txt)
  --input=<input-type>  Datatypes in input belong to <input-type>.
                        valid values:   mysql   (for mysql)
                                        mssql   (for ms-sql)
                                        pgsql   (for postgresql)

Converts xml data input from Dia to sql statements. If outfile.sql is not
specified the sql statements will be printed to STDOUT.

Edit dia2sql.pl and change the configuration at top of the Perl script.
Make sure you have defined the right database (MySQL or PostgreSQL) for
SQL output.

E_O_USAGE

exit(1);
}
