#!/usr/bin/perl
# $Id: enum_parse.pl,v 1.6 2001/08/12 23:20:01 fkr Exp $
# (c) SMDP-Group - http://smdp.sourceforge.net/
# enum_parse.pl
# or
# 1 pack of cigs, 6 cans of coke and 1 ham and pepperoni sandwich worth of perl
# knowlecge

use Getopt::Long;

$VERSION= "enum_parse.pl 0.01";


$DEBUG = 0;                 # print debugging output (currently unused)
$sep = ",";                 # output seperator
$infile = "globals.h";      # id3lib header file with tag info
$key = "ID3_FrameID";       # enum name to search for
$enum_prefix = "ID3FID_";   # prefix to enum values

$results = &GetOptions("seperator=s" => \$seperator,   # change seperator
                       "prefix=s" => \$prefix,         # use prefix "s"
                       "suffix=s" => \$suffix,         # use suffix "s"
		       "infile=s" => \$infile,	       # overwrite infile
                       "key=s" => \$key_val,           # enum search key
                       "debug", \$debug,	       # we want debug-mode
		       "h", \$usage,                   
                       "help", \$usage);               

if(!($results) || defined($usage)){
    &usage();
}

if(defined($seperator)){
    $sep = $seperator;
}

if(defined($prefix)){
    $pre = $prefix;
}

if(defined($suffix)){
    $suf = $suffix;
}

if(defined($key_val)){
    $key = $key_val;
}

if(defined($debug)){
    $DEBUG = 1;
}

if($DEBUG){
    print STDOUT "[debug] opening $infile as source-file.\n";
}

open(ID3,"< $infile") || die "Unable to open $infile in read-mode: $!\n";

$date = `date`; chop($date);
print STDOUT "# Created by: " . $VERSION . " ( ". $date . " )\n\n";


while (<ID3>) {
    chomp;
    if(index($_, $key) != -1){
        # the joy of slop..
        # pass through the found line and the following which should only 
        # contain a '{'
        $_ = <ID3>;
	$_ = <ID3>;
        
        for(; substr($_, 0, 2) ne "};"; $_ = <ID3>) {
            /$enum_prefix[A-Z1-9]*/;
            $name = $&;
            # Pull out the description comment
            /\/\*\*< .* \*\//;
            # Chop off the /**< and */ around the description
            $desc = substr($&, 5,  length($&) - 8);
            
            print STDOUT $pre . $name . $sep . $desc . $suf . "\n";
        }
    }
}

if($DEBUG){
	print STDOUT "finished with $infile.\n";
}
close(ID3);

    
sub usage() {
    print STDOUT <<E_O_USAGE;

This is: $VERSION
Usage:  ./enum_parse.pl [ options ]

Currently pretty restrictive, but for now:

  --prefix=s        print 's' before each output line [NONE]
  --suffix=s        print 's' after each line [NONE]
  --seperator=s     use 's' as the seperator [,]
  --key-val=s       search for enum typedef of 's'
                    [ ID3_TagType | ID3_FieldID | ID3_FrameID ]
  --infile=s        overwrite the default infile
  --debug           turn debug-mode on
  -h, --help        print this usage info
  

E_O_USAGE

exit(1);
}


# EOF
