#!/bin/sh
# (c) SMDP-Group - http://smdp.sourceforge.net
# general questions: smdp-devel@lists.sourceforge.net
# specific question about setup-db.sh: fkr@grummel.net
# $Id: setup-db.sh,v 1.17 2001/08/16 22:57:38 fkr Exp $

sh="/bin/sh"

VERSION="setup-db.sh 0.01"
config_file="smdp.conf"

# catch command-line-args
while [ $# -gt 0 ]
do
    case "$1" in
        --help|-h)
       		usage=t
			break;;
		--mysql-parser=*)
			database_parser_m=`echo $1 | sed 's/^--mysql-parser=//'`
			;;
		--psql-parser=*)
       		database_parser_p=`echo $1 | sed 's/^--psql-parser=//'`
      		;;
		--with-mysql=*)
			with_mysql=`echo $1 | sed 's/^--with-mysql=//'`
			;;
		--with-pgsql=*)
			with_pgsql=`echo $1 | sed 's/^--with-pgsql=//'`
			;;
		--db-user=*)
			database_user=`echo $1 | sed 's/^--db-user=//'`
			;;
		--db-pass=*)
			database_pass=`echo $1 | sed 's/^--db-pass=//'`
			;;
		--db-schema=*)
			database_schema=`echo $1 | sed 's/^--db-schema=//'`
			;;
		--db-host=*)
			database_host=`echo $1 | sed 's/^--db-host=//'`
			;;
		--db-name=*)
			database_name=`echo $1 | sed 's/^--db-name=//'`
			;;
		--with-cat=*)
			cat=`echo $1 | sed 's/^--with-cat=//'`
			;;
		--devel)
			devel=t
			;;
		--with-config-file=*)
			config_file=`echo $1 | sed 's/^--with-config-file=//'`
			write_config=t
			;;
	esac
	shift
done

if [ "$usage" ]; then
    echo ""
    echo "This is: ${VERSION}"
    echo "Usage: setup-db.sh [option]"
    echo ""
    echo "Where option is one of:"
    echo "  -h                    print this usage statement"
    echo "  --mysql-parser=/path/to/mysql-binary"
    echo "  --psql-parser=/path/to/psql-binary"
	echo "  --with-mysql=[ no ]   to disable mysql"
	echo "  --with-pgsql=[ no ]   to disable postgresql"
    echo "  --db-user=[ username for database ]"
    echo "  --db-pass=[ password for the user ]"
    echo "  --db-schema=s         (base or ext)"
    echo "  --db-host=s           (defaults to localhost)"
    echo "  --with-cat=/path/to/cat-binary"
    echo "  --write-config        writes config-file to ./smdp.conf"
    echo "  --with-config-file=s  name of config-file"
    echo ""
    exit 0
fi

echo ""
echo ""
echo "Welcome to the SMDP database initalization..."
echo ""
echo ""

# display warning in develope-mode
if [ "$devel" ]; then
    echo "ATTENTION: running in develope-mode!"
    echo ""
fi

mysql_schema="MySQL/smdp-schema.sql"
psql_schema="PostgreSQL/smdp-schema.sql"
id3_frames_sql="general-sql/id3-frames.sql"

# let's find out where 'cat' is
if [ "$cat" == "" ]; then
	if [ -e "/bin/cat" ]; then
		cat="/bin/cat"
	elif [ -e "/usr/bin/cat" ]; then
		cat="/usr/bin/cat"
	elif [ -e "/usr/local/bin/cat" ]; then
		cat="/usr/local/bin/cat"
	fi
fi

# now some nasty if-else stuff, to determine wether
# we init a postgresql or mysql
if [ "$with_pgsql" != "no" ]; then
	if [ "$database_parser_p" != "" ]; then
		database_type="postgresql"
	fi
fi
if [ "$with_mysql" != "no" ]; then
	if [ "$database_parser_m" != "" ]; then
		database_type="mysql"
	fi
fi


# and now...lot's of questions
echo ""
echo "I'm now going to ask you several questions, possible answers are"
echo "displayed in brackets."
echo ""

if [ "$database_user" == "" ]; then
	echo ""
	echo "What's the username of the database-user to use during initialization?"
	echo -n "[default: smdp]: "
	read database_user
	# default to 'smdp' if username is empty
	if [ "$database_user" == "" ]; then
		database_user="smdp"
	fi
fi

if [ "$database_pass" == "" ]; then
	echo ""
	echo "What's the password for the database-user?"
	echo -n "[won't be echoed]: "
	stty -echo # turning of echo 
	read database_pass
	stty echo # turning it back on

	echo ""
fi

# if database_type is not set, ask for it....
if [ "$database_type" == "" ]; then
	echo ""
	echo "On which RDBMS-System do you want to install SMDP?"
	echo -n "[mysql] || [postgresql]: "
	read database_type
	echo ""
fi

# on which host do we initialize?
if [ "$database_host" != "" ]; then
	database_host="-h $database_host"
fi

# if the user wants a different database, rebuild sql-scripts
if [ "$database_name" != "" ]; then
	if [ -e "build-schema.sh" ]; then
		build_schema="build-schema.sh"
	else
		echo ""
		echo "Can't find build-schema.sh which is needed for rebuilding sql-scripts."
		echo ""
		exit;
	fi
	if [ "$devel" ]; then
		${sh} ${build_schema} --db-name=${database_name} --build-type=devel
	else
		${sh} ${build_schema} --db-name=${database_name} --build-type=production
	fi
else
	database_name=smdp
	if [ "$devel" ]; then
        if [ -e "build-schema.sh" ]; then
		        build_schema="build-schema.sh"
	    else
			echo ""
	        echo "Can't find build-schema.sh which is needed for rebuilding sql-scripts."
	        echo ""
	        exit;
	    fi
		${sh} ${build_schema} --build-type=devel
	fi
fi


# now we should have all informations
if [ "$database_type" == "mysql" ]; then
	# mysql-stuff
	echo ""
	echo "Ok, we're working on a mysql-database."
	echo ""

	if [ "$database_parser_m" != "" ]; then
		database_parser=$database_parser_m
	fi
	
	# parser is not set yet. try to find it!
	if [ "$database_parser" == "" ]; then
		echo "Location of the 'mysql'-binary unkown. Trying to locate it..."
		if [ -e "/usr/bin/mysql" ]; then
			database_parser="/usr/bin/mysql"
		elif [ -e "/usr/local/bin/mysql" ]; then
			database_parser="/usr/local/bin/mysql"
		fi
		echo ""
	fi
	
	echo ""
	echo "Setting up the database...."
	${database_parser} -u ${database_user} -p${database_pass} ${database_host} < ${mysql_schema}
	echo ""

elif [ "$database_type" == "postgresql" ]; then
	# postgresql-stuff

	if [ "$database_parser_p" != "" ]; then
    		database_parser=$database_parser_p
    	fi

	# parser is not set yet. try to find it!
	if [ "$database_parser" == "" ]; then
		echo "Location of the 'psql'-binary unkown. Trying to locate it..."
		if [ -e "/usr/bin/psql" ]; then
            		database_parser="/usr/bin/psql"
        	elif [ -e "/usr/local/bin/psql" ]; then
            		database_parser="/usr/local/bin/psql"
       		fi
	echo ""
 	fi
	
	echo "You will be prompted for the password for the postgresql-user ${database_user}."
	echo ""
	echo ""
    echo "Setting up the database...."
	${cat} ${psql_schema} | ${database_parser} -U ${database_user} ${database_host} -d template1
	echo ""
fi

if [ "$write_config" ]; then
	
	# if database_host is empty, direct it to localhost
	if [ "$database_host" == "" ]; then
		database_host="localhost"
	fi
	
	# writes config to $config_file
	echo ""
	echo "Writing config to $config_file"
	echo "# Created by: setup-db.sh ( `date` ) " > $config_file
	echo "" >> $config_file
	echo "DB_TYPE $database_type" >> $config_file 
	echo "DB_HOST $database_host" >> $config_file
	echo "DB_NAME $database_name" >> $config_file
	echo "DB_USER $database_user" >> $config_file 
	echo "DB_PASS $database_pass" >> $config_file 
	echo ""
	echo "Please don't forget to chmod/chown $config_file correctly!"
	echo ""
fi


echo ""
echo ""
echo "Database is now initialized. Have fun with SMDP."
echo ""
